// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/user.dart';

abstract class RoomEvent extends Event {
  final Room room;
  final User sender;
  final DateTime time;

  @override
  final SentState sentState;

  RoomEvent(RoomEventArgs args)
    : room = args.room,
      sender = args.sender,
      time = args.time,
      sentState = args.sentState,
      super(args.id, args.unsigned);

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'sender': sender.id.toJson(),
      'origin_server_ts': time.millisecondsSinceEpoch
    });
}

class RoomEventArgs {
  final EventId id;
  final Room room;
  final User sender;
  final DateTime time;
  final UnsignedData unsigned;
  final SentState sentState;

  RoomEventArgs({
    this.id,
    this.room,
    this.sender,
    this.time,
    this.unsigned,
    this.sentState = SentState.sent
  });

  RoomEventArgs copyWith({
    EventId id,
    Room room,
    User sender,
    DateTime time,
    UnsignedData unsigned,
    SentState sentState
  }) {
    if (id == null) {
      id = this.id;
    }

    if (room == null) {
      room = this.room;
    }

    if (sender == null) {
      sender = this.sender;
    }

    if (time == null) {
      time = this.time;
    }

    if (unsigned == null) {
      unsigned = this.unsigned;
    }

    if (sentState == null) {
      sentState = this.sentState;
    }

    return RoomEventArgs(
      id: id,
      room: room,
      sender: sender,
      time: time,
      unsigned: unsigned,
      sentState: sentState
    );
  }
}